@@ -48,6 +48,14 @@
 #define MAILBOX_ORG_CALDAV   "https://dav.mailbox.org/caldav"
 #define MAILBOX_ORG_CARDDAV  "https://dav.mailbox.org/carddav"
 
+/* mail.ru
+ * See: https://help.mail.ru/cloud_web/app/webdav/#linux
+ *      https://help.mail.ru/calendar-help/synchronization/about/
+ */
+#define MAIL_RU_HOSTNAME     "mail.ru"
+#define MAIL_RU_WEBDAV       "https://webdav.cloud.mail.ru"
+#define MAIL_RU_CALDAV       "https://calendar.mail.ru"
+
 struct _GoaDavClient
 {
   GObject parent_instance;
@@ -738,10 +746,11 @@ g_resolver_lookup_service_cb (GResolver    *resolver,
     }
   else if (!data->base_domain_fallback)
     {
+      g_autofree char *domain_lowercase = g_utf8_casefold (data->domain, -1);
       const char *base_domain = NULL;
 
-      base_domain = soup_tld_get_base_domain (data->domain, NULL);
-      if (base_domain == NULL || g_strcmp0 (data->domain, base_domain) == 0)
+      base_domain = soup_tld_get_base_domain (domain_lowercase, NULL);
+      if (base_domain == NULL || g_strcmp0 (domain_lowercase, base_domain) == 0)
         {
           g_task_return_error (task, g_steal_pointer (&error));
           return;
@@ -1176,13 +1185,6 @@ dav_client_discover_iterate (GTask *task)
         {
           g_autofree char *nc_uri = NULL;
 
-          /* Start with Nextcloud, everyone's favourite sort of compliant DAV server, since
-           * we can return early if we recognize the path.
-           */
-          nc_uri = g_uri_resolve_relative (discover->uri, "remote.php/dav", G_URI_FLAGS_NONE, NULL);
-          g_queue_push_tail (&discover->candidates,
-                             goa_dav_config_new (GOA_SERVICE_TYPE_WEBDAV, nc_uri, NULL));
-
           g_queue_push_tail (&discover->candidates,
                              goa_dav_config_new (GOA_SERVICE_TYPE_CALDAV, discover->uri, NULL));
           g_queue_push_tail (&discover->candidates,
@@ -1190,6 +1192,12 @@ dav_client_discover_iterate (GTask *task)
           g_queue_push_tail (&discover->candidates,
                              goa_dav_config_new (GOA_SERVICE_TYPE_WEBDAV, discover->uri, NULL));
 
+          /* Fallback to making an OPTIONS request to a known path for Nextcloud/ownCloud
+           */
+          nc_uri = g_uri_resolve_relative (discover->uri, "remote.php/dav", G_URI_FLAGS_NONE, NULL);
+          g_queue_push_tail (&discover->candidates,
+                             goa_dav_config_new (GOA_SERVICE_TYPE_WEBDAV, nc_uri, NULL));
+
           discover->uri_fallback = TRUE;
           dav_client_discover_iterate (task);
         }
@@ -1221,12 +1229,13 @@ static gboolean
 dav_client_discover_preconfig (DiscoverData *discover,
                                GUri         *uri)
 {
-  const char *host = NULL;
+  g_autofree char *host = NULL;
   const char *base_domain = NULL;
 
   g_assert (discover != NULL);
 
-  host = g_uri_get_host (uri);
+  if (g_uri_get_host (uri))
+    host = g_utf8_casefold (g_uri_get_host (uri), -1);
   if (host != NULL)
     base_domain = soup_tld_get_base_domain (host, NULL);
 
@@ -1247,7 +1256,7 @@ dav_client_discover_preconfig (DiscoverData *discover,
       || g_strcmp0 (base_domain, "mailbox.org") == 0)
     {
       g_queue_push_tail (&discover->candidates,
-                         goa_dav_config_new (GOA_SERVICE_TYPE_CALDAV, MAILBOX_ORG_WEBDAV, NULL));
+                         goa_dav_config_new (GOA_SERVICE_TYPE_CALDAV, MAILBOX_ORG_CALDAV, NULL));
       g_queue_push_tail (&discover->candidates,
                          goa_dav_config_new (GOA_SERVICE_TYPE_CARDDAV, MAILBOX_ORG_CARDDAV, NULL));
       g_queue_push_tail (&discover->candidates,
@@ -1256,6 +1265,17 @@ dav_client_discover_preconfig (DiscoverData *discover,
       return TRUE;
     }
 
+  if (g_strcmp0 (host, "mail.ru") == 0
+      || g_strcmp0 (base_domain, "mail.ru") == 0)
+    {
+      g_queue_push_tail (&discover->candidates,
+                         goa_dav_config_new (GOA_SERVICE_TYPE_CALDAV, MAIL_RU_CALDAV, NULL));
+      g_queue_push_tail (&discover->candidates,
+                         goa_dav_config_new (GOA_SERVICE_TYPE_WEBDAV, MAIL_RU_WEBDAV, NULL));
+
+      return TRUE;
+    }
+
   return FALSE;
 }
 
