@@ -1593,10 +1593,14 @@ goa_oauth2_provider_add_account (GoaProvider         *provider,
       data->dialog = GOA_PROVIDER_DIALOG (parent);
       data->flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (data->dialog),
                                                          "goa-auth-flow-flags"));
-      g_object_set_data_full (G_OBJECT (task),
-                              "goa-provider-dialog",
-                              g_object_ref (data->dialog),
-                              g_object_unref);
+      /* The signal handler holds the reference to @task.
+       */
+      g_signal_connect_object (data->dialog,
+                               "notify::state",
+                               G_CALLBACK (add_account_action_cb),
+                               g_object_ref (task),
+                               G_CONNECT_DEFAULT);
+      g_object_set_data (G_OBJECT (task), "goa-provider-dialog", parent);
       add_account_action_cb (data->dialog, NULL, task);
     }
   else
@@ -1767,10 +1771,16 @@ goa_oauth2_provider_refresh_account (GoaProvider         *provider,
   if (GOA_IS_PROVIDER_DIALOG (parent))
     {
       data->dialog = GOA_PROVIDER_DIALOG (parent);
-      g_object_set_data_full (G_OBJECT (task),
-                              "goa-provider-dialog",
-                              g_object_ref (data->dialog),
-                              g_object_unref);
+      data->flags = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (data->dialog),
+                                                         "goa-auth-flow-flags"));
+      /* The signal handler holds the reference to @task.
+       */
+      g_signal_connect_object (data->dialog,
+                               "notify::state",
+                               G_CALLBACK (refresh_account_action_cb),
+                               g_object_ref (task),
+                               G_CONNECT_DEFAULT);
+      g_object_set_data (G_OBJECT (task), "goa-provider-dialog", parent);
       refresh_account_action_cb (data->dialog, NULL, task);
     }
   else
@@ -2079,7 +2089,6 @@ goa_oauth2_provider_ensure_credentials_sync (GoaProvider   *provider,
   gchar *identity = NULL;
   gboolean force_refresh = FALSE;
 
- again:
   access_token = goa_oauth2_provider_get_access_token_sync (self,
                                                             object,
                                                             force_refresh,
@@ -2089,28 +2098,6 @@ goa_oauth2_provider_ensure_credentials_sync (GoaProvider   *provider,
   if (access_token == NULL)
     goto out;
 
-  identity = goa_oauth2_provider_get_identity_sync (self,
-                                                    access_token,
-                                                    NULL, /* out_presentation_identity */
-                                                    cancellable,
-                                                    error);
-  if (identity == NULL)
-    {
-      /* OK, try again, with forcing the locally cached credentials to be refreshed */
-      if (!force_refresh)
-        {
-          force_refresh = TRUE;
-          g_free (access_token); access_token = NULL;
-          g_clear_error (error);
-          goto again;
-        }
-      else
-        {
-          goto out;
-        }
-    }
-
-  /* TODO: maybe check with the identity we have */
   ret = TRUE;
   if (out_expires_in != NULL)
     *out_expires_in = access_token_expires_in;
